import React, { useEffect, useMemo, useState } from 'react'
import { Select, message } from 'antd'
import { getUserSelector } from '../../../../../apis/process'

const UserSelect = (props) => {

  const userID = window?.globalConfig?.userInfo?.OID || 1
  const { value, onChange, schema, addons } = props
  const { disabled, presetValue, isMultiple, placeholder, role, isStoreID, isMySite } = schema
  const property = isStoreID ? 'userID' : 'userName'
  const [pullDown, setPullDown] = useState([])
  const [searchValue, setSearchValue] = useState('')

  const valueShow = useMemo(() => {
    if (isMultiple) {
      return value ? value.split(',') : []
    }
    return value || null
  }, [value, isMultiple])

  const onSearch = (value) => {
    setSearchValue(value)
  }

  const selectFocus = async () => {
    let time = new Date().getTime() + Math.random()
    const { code, data, msg } = await getUserSelector(role, isMySite ? userID : false, time)
    if (code === 0) {
      if (data) {
        setPullDown(data.dropdownList.map((v, i) => {
          let length = v.groupName?.length
          return {
            label: (
              <div style={{ display: 'flex', justifyContent: 'space-between' }}>
                <span>{v.userName}</span>
                <span style={{ fontSize: '12px', color: '#b4b1b1', }}>{length > 20 ? `...${v.groupName.slice(length - 20, length)}` : v.groupName}</span>
              </div>
            ),
            userName: v.userName,
            value: v[property],
            groupName: v.groupName
          }
        }))
      } else {
        setPullDown([])
      }
    } else {
      message.info(msg)
    }
  }

  const selectChange = (value) => {
    setSearchValue('')
    if (addons) {
      onChange(isMultiple ? value.join(',') : value)
    }
  }

  useEffect(() => {
    if (addons) {
      addons.setValue(addons.dataPath, presetValue)
    } else {
      onChange(presetValue)
    }
  }, [presetValue])

  useEffect(() => {
    if (isStoreID) {
      selectFocus()
    }
  }, [])

  return (
    <Select
      mode={isMultiple ? 'multiple' : null}
      placeholder={disabled ? (placeholder || '') : (placeholder || '点击选择人员')}
      value={valueShow}
      disabled={disabled}
      showArrow={!disabled}
      showSearch={true}
      onSearch={onSearch}
      searchValue={searchValue}
      style={{ width: '100%' }}
      onFocus={selectFocus}
      filterOption={(input, option) => (option?.userName || '').toLowerCase().includes(input.toLowerCase())}
      options={pullDown}
      onChange={selectChange}
    >
    </Select>
  )

}

export default UserSelect